#####
#
# 02/27/2011
# Clint Kitson - EMC vSpecialist - @clintonskitson 
# EMC Community Network - Everything VMware at EMC
# https://community.emc.com/community/connect/everything_vmware?view=overview
#
# 03/30/11 Version 1.1 (suggested currently for non-production use and lab use only, provided as-is)
#
# get_symcli_perf.ps1 - collect and process statistics from a EMC Symmetrix storage array
#
# UPDATES
# 02/28/11 updated outcsv to ensure same datetime is used during sampling
# 03/30/11 updated to allow for multiple symm's from the same SE
#
# PARAMETERS
# -paint = adding a -paint will tell the script to refresh stats to same window and refresh stats continuously until interrupted
# -interval (15 seconds default) = the sampling period to collect performance stats
# -metrics = which metrics (columns) to be returned, * will return all columns
#
# -symid - return only information from this sym (not needed is solutions enabler is attached to only one sym)
# -lunid  Return only specific information based on this LUNID
# -dev_name  return only specific devices making up the meta based on this device name
# -sleep (5 seconds default) = amount of time to wait in between set of performance samplings
# -outcsv = output to a csv file
#
# EXAMPLE
# .\get_symcli_perf.ps1 -symclihost symclihost -username user -password pass -paint
# .\get_symcli_perf.ps1 -symclihost symclihost -username user -password pass -paint -outcsv test.csv
#
#####

param($symclihost,[string]$username,[string]$password,[string]$lunid,[string]$dev_name,[switch]$paint,[string]$interval,$metrics,[string]$sleep,$outcsv,$symid)

if (!($symclihost -and $username -and $password)) { 
    write "must specify -symclihost -username -password"
    exit 
}


#$symclihost = "10.5.132.65"
#$lunid = "60000970000194900728533030304333"


if (!$interval) { $interval = 15 }
if (!$metrics) { [array]$metrics = "name,r_per_second,w_per_second,io_per_second,kb_r_per_second,kb_w_per_second,r_cache_hit_pct,w_cache_hit_pct,rw_cache_hit_pct,sequential_r_pct,wp_tracks".split(',') }
if (!$sleep) { $sleep = 5 }

function xml_to_hash ($xmlobj,$x) {
    $tmphash = @{}
    $xmlobj | gm * | where { $_.MemberType -eq "Property" } | select name | %{ $name=$_.name;$tmphash[$name] = $xmlobj.$name }
    $tmphash.order = $x
    return $tmphash
}

for ($i) {

    $path = (Split-Path -parent $MyInvocation.MyCommand.Definition) + "\"
    $spath = $path + "ssh_function.ps1"
    . $spath

    if ($symclihost -and $username -and $password) {
    	New-SshSession $username $password $symclihost

        $listdir = invoke-ssh "symcfg list -dir ALL -v -output xml;uname -a" "GNU/Linux"
        $xmlListDir = [xml]$listdir[3..($listdir.count-4)]
        
        if (!$symid) {
            if (!$xmlListDir.SymCLI_ML.Symmetrix.Symm_Info.symid) { 
                write "Multiple Symms detected, you must specify symid with -symid";
                $xmlListDir.SymCLI_ML.Symmetrix | %{ $_.Symm_Info.symid }
                exit 
                
            } else {
                $symid = $xmlListDir.SymCLI_ML.Symmetrix.Symm_Info.symid
            }
        }
        
        $director = $xmlListDir.SymCLI_ML.Symmetrix.Director

        $listlogins = invoke-ssh "symmask -sid $symid list logins -v -output xml;uname -a" "GNU/Linux"
        $xmlListlogins = [xml]$listlogins[3..($listlogins.count-4)]
        $Devmask_Login_Record = $xmlListLogins.SymCLI_ML.Symmetrix.Devmask_Login_Record

        $listwwn = invoke-ssh "symdev list -wwn -output xml;uname -a" "GNU/Linux"
        $xmlListWwn = [xml]$listwwn[3..($listwwn.count-4)]
        $device = $xmlListWwn.SymCLI_ML.Symmetrix.Device

        if ($lunid) { $dev_name = ($device | where { $_.wwn -eq $lunid }).dev_name } 
    
        if ($dev_name) {
            #collect valid device names we want to target
            $showtdev = invoke-ssh "symdev show $dev_name -output xml;uname -a" "GNU/Linux"
            $xmlShowTdev = [xml]$showtdev[3..($showtdev.count-4)]
            $meta_device = $xmlShowTdev.SymCLI_ML.Symmetrix.Device.Meta.Meta_Device        
            $arrDevName = ($meta_device | select dev_name | %{ $_.dev_name })
            $total_capacity = $xmlShowTdev.SymCLI_ML.Symmetrix.Device.Meta.Total_Capacity
        }
        Remove-SshSession 
    } else { write "missing -username -password -symclihost"; exit }


    $output = @{}
    $output.symid += $xmlListDir.SymCLI_ML.Symmetrix.Symm_Info.symid
    $output.directors += $xmlListDir.SymCLI_ML.Symmetrix.Director
    $output.Devmask_Login_Record += $Devmask_Login_Record[0].login | where { $_.logged_in -eq 'yes' -and $_.on_fabric -eq 'Yes' }
    $output.device += $device
    $output.dev_name += $dev_name
    $output.meta_device += $meta_device
    $output.total_capacity += $total_capacity

    $scpath = $path + "\exec_symcli_cmd.ps1"

    $job1 = {
                set-location $args[0];
                $cmd = "&'"+$args[5]+"' -symclihost "+$args[1]+" -username "+$args[2]+" -password "+$args[3]+" -interval "+$args[6]+" -statcmd '"+$args[4]+"'"; invoke-expression $cmd
                }  
    $job2 = {
                set-location $args[0];
                $cmd = "&'"+$args[5]+"' -symclihost "+$args[1]+" -username "+$args[2]+" -password "+$args[3]+" -interval "+$args[6]+" -statcmd '"+$args[4]+"'"; invoke-expression $cmd
                }  
    $job3 = {
                set-location $args[0];
                $cmd = "&'"+$args[5]+"' -symclihost "+$args[1]+" -username "+$args[2]+" -password "+$args[3]+" -interval "+$args[6]+" -statcmd '"+$args[4]+"'"; invoke-expression $cmd
                }  

    get-job | stop-job
    get-job | remove-job
    
    $statcmd = "-type REQUESTS"
    start-job -argumentlist (get-location), $symclihost, $username, $password, $statcmd, $scpath, $interval -name job1 $job1 | out-null
    
    $statcmd = "-type DISK -sid "+$symid
    start-job -argumentlist (get-location), $symclihost, $username, $password, $statcmd, $scpath, $interval, $symid -name job2 $job2 | out-null
    
    $statcmd = "-dir ALL"
    start-job -argumentlist (get-location), $symclihost, $username, $password, $statcmd, $scpath, $interval -name job3 $job3 | out-null

    $count=1
    do {
        $count++
        sleep 1
    }
    while ((get-job | where {$_.State -ne 'Completed'}) -and ($count -lt 10+$interval))

    if ($count -eq 10+$interval) {
        get-job | where {$_.State -ne 'Completed'} | stop-job
        write "quit becuase it took too long"
        exit
    }

    $x = 0
    $stats = @{}

    $job3output = @()
    $job3output = get-job -name job3 | receive-job 
    $xmlStatDirs = [xml]$job3output[3..($job3output.count-4)]
    $arrStatDirs = @()
    $arrStatDirs = $xmlStatDirs.SymCLI_ML.Statistics[1].Dir_Request
    [array]$arrStatDirs = $arrStatDirs | sort director
    if ($arrStatDirs) { $arrStatDirs | %{ $stats["Director "+$_.director] = xml_to_hash $_ $x;$x++ } }

    $job1output = @()
    $job1output = get-job -name job1 | receive-job 
    $xmlStatRequests = [xml]$job1output[3..($job1output.count-4)]
    $arrStatRequests = @()
    $arrStatRequests = $xmlStatRequests.SymCLI_ML.Statistics[1].Request
    [array]$arrStatRequests = $arrStatRequests | sort dev_name    
    
    #if we chose a lunid or dev_name then filter the devices down to what we want to see

    if ($arrDevName) { 
        [array]$arrStatRequests = $arrStatRequests | where {$arrDevName -contains $_.dev_name}
        if ($arrStatRequests) { $arrStatRequests | %{ $stats["Device "+$_.dev_name] = xml_to_hash $_ $x;$x++ } }
    } elseif (!$lunid) {
        if ($arrStatRequests) { $arrStatRequests | %{ $stats["Device "+$_.dev_name] = xml_to_hash $_ $x;$x++ } }
    }

    $job2output = @()
    $job2output = get-job -name job2 | receive-job 
    $xmlStatDisks = [xml]$job2output[3..($job2output.count-4)]
    $arrStatDisks = @()
    $arrStatDisks = $xmlStatDisks.SymCLI_ML.Statistics[1].Disk
    [array]$arrStatDisks = $arrStatDisks | sort disk_name
    if ($arrStatDisks) { $arrStatDisks | %{ $stats["Disk "+$_.disk_name] = xml_to_hash $_ $x;$x++ } }

    #$temp = invoke-ssh "symdev list -datadev -v  -output xml;uname -a" "GNU/Linux"
    #$temp = invoke-ssh "symdev list -meta -v -output xml;uname -a" "GNU/Linux"

    $outstats = @{}
    $statstemp = @{}
    $allstats = @()
    foreach ($stat in $stats.keys) {

        $statkey = $stat
        $statnames = $stats[$stat].keys | where { $_ -match 'per_|hit_|order|pct|tracks'}
        $stat = $stats[$stat]

        $row = @{}
        $rowcalc = @{}

        if (!$statstemp[$statkey]) { $statstemp[$statkey] = @{} }
                    
        # throw original stats into new statstemp hash
        $statnames | %{
            $name = $_
            if ($stat.$name -ne 'N/A') { $rowcalc.$name = $stat.$name } else { $rowcalc.$name = "" }
        }     

        # CHECK FOR 0's
        $rowcalc2 = @{}
        $rowcalc.keys | %{ if ($rowcalc.$_ -eq 0) { $rowcalc2.$_ = $nul } else { $rowcalc2.$_ = $rowcalc.$_ } } 
        $rowcalc = $rowcalc2

        $outstats[$statkey] = $rowcalc 
        $allstats += $statnames
    }

    [array]$allstats = $allstats | select -unique
    
    $outdate = get-date -f s
    $out_rollout = %{
        foreach ($itemname in $statstemp.keys) {              
            $obj = new-object PSObject
            $obj | Add-member Noteproperty -name Name -value $itemname
            $obj | Add-member Noteproperty -name Date -value $outdate

            $allstats | %{
                   $obj | Add-member Noteproperty -name $_ -value $outstats[$itemname][$_] 
            }
            write $obj
        }
    }

    if ($outcsv) {
        $csvpath = $path + $outcsv
        $csvcols = ""
        $coldate = get-date -f o
        write "Writing output to $csvpath with collection date of $coldate"

        [array]$newstats = "Date","Name"
        $allstats2 = @()
        [array]$allstats2 = $newstats + $allstats

        if(test-path $csvpath) {
            $out_rollout |  %{ $csvout="";$trow=$_; $allstats2 | %{ $tt = $trow.$_; $csvout += "$tt," };$csvout.substring(0,$csvout.length-1) } | out-file -append $csvpath -encoding ascii
        
        } else {
            #write the header and the data lines
            $allstats2 | %{ $csvcols += $_+"," }
            $csvcols.trim(',') | out-file $csvpath -encoding ascii
            $out_rollout |  %{ $csvout="";$trow=$_; $allstats2 | %{ $tt = $trow.$_; $csvout += "$tt," };$csvout.substring(0,$csvout.length-1) } | out-file -append $csvpath -encoding ascii

        }
    } else {            
        if ($paint -and $metrics) {
            clear;$out_rollout | sort order | select $metrics | ft *
        } elseif ($paint) {
            clear;$out_rollout | sort order | select * | ft *
        } elseif ($metrics) {
            $out_rollout | sort order | select $metrics
            exit
        } else {
            $out_rollout | sort order | select *
            exit
        }
    }
    sleep $sleep
}

